
//////////////////////////////////////////////////////////////
//
// Source file for MainWindowMainWindow
//
//    This class is a ViewKit VkWindow subclass
//
//
// Normally, very little in this file should need to be changed.
// Create/add/modify menus using the builder.
//
// Try to restrict any changes to the bodies of functions
// corresponding to menu items, the constructor and destructor.
//
// Restrict changes to those sections between
// the "//--- Start/End editable code block" markers
//
// Doing so will allow you to make changes using the builder
// without losing any changes you may have made manually
//
// Avoid gratuitous reformatting and other changes that might
// make it difficult to integrate changes made using the builder
//////////////////////////////////////////////////////////////
#include "MainWindowMainWindow.h"
#include <stdio.h>
#include <Vk/VkApp.h>
#include <Vk/VkFileSelectionDialog.h>
#include <Vk/VkSubMenu.h>
#include <Vk/VkRadioSubMenu.h>
#include <Vk/VkMenuItem.h>
#include "MainForm.h"
//---- Start editable code block: headers and declarations

#include <bstring.h>
#include <BVN/Reactor.h>
#include <BVN/Acceptor.h>
#include <BVN/SOCK_Acceptor.h>
#include "BV.h"
#include "BV_Svc_Handler.h"
#include "BV_Session_Control.h"
#include "BV_Tango_Control.h"
#include "BV_AudioApp.h"
#include "BV_VideoApp.h"

#define BV_PORT 5000

extern XtWorkProcId ctrlWorkProcHandle;

//---- End editable code block: headers and declarations



// These are default resources for widgets in objects of this class
// All resources will be prepended by *<name> at instantiation,
// where <name> is the name of the specific instance, as well as the
// name of the baseWidget. These are only defaults, and may be overriden
// in a resource file by providing a more specific resource name

String  MainWindowMainWindow::_defaultMainWindowMainWindowResources[] = {
        "*title:  BuenaVista Videoconferencing",
        "*filePane.labelString:  File",
        "*filePane.mnemonic:  F",
        "*resetButton.labelString:  Reset Session",
        "*resetButton.mnemonic:  R",
        "*resetButton.accelerator:  Ctrl<Key>R",
        "*resetButton.acceleratorText:  Ctrl+R",
        "*exitButton.labelString:  Exit",
        "*exitButton.mnemonic:  x",
        "*exitButton.accelerator:  Ctrl<Key>Q",
        "*exitButton.acceleratorText:  Ctrl+Q",
        "*editPane.labelString:  Edit",
        "*editPane.mnemonic:  E",
        "*optionsButton.labelString:  Options",
        "*optionsButton.mnemonic:  O",
        "*optionsButton.accelerator:  Ctrl<Key>O",
        "*optionsButton.acceleratorText:  Ctrl+O",
        "*viewPane.labelString:  View",
        "*viewPane.mnemonic:  V",
        "*audioPanelButton.labelString:  Audio Panel",
        "*audioPanelButton.mnemonic:  P",
        "*statButton.labelString:  Statistics",
        "*statButton.mnemonic:  S",
        "*statButton.accelerator:  Ctrl<Key>S",
        "*statButton.acceleratorText:  Ctrl+S",
        "*toolsPane.labelString:  Tools",
        "*toolsPane.mnemonic:  T",
        "*chatButton.labelString:  Text Chat",
        "*chatButton.mnemonic:  C",
        "*helpPane.labelString:  Help",
        "*helpPane.mnemonic:  H",
        "*help_overview.labelString:  Overview",
        "*help_overview.mnemonic:  O",
        "*directoryPane.labelString:  Directory",
        "*entry.labelString:  ",
        "*usersButton.labelString:  Active Users",
        "*usersButton.mnemonic:  U",
        "*confButton.labelString:  Conferences",
        "*confButton.mnemonic:  C",
        (char*)NULL
};


//---- Class declaration

MainWindowMainWindow::MainWindowMainWindow( const char *name,
                                            ArgList args,
                                            Cardinal argCount) : 
                                      VkWindow (name, args, argCount) 
{
    // Load any class-default resources for this object

    setDefaultResources(baseWidget(), _defaultMainWindowMainWindowResources  );


    // Create the view component contained by this window

    _mainForm= new MainForm("mainForm",mainWindowWidget());


    XtVaSetValues ( _mainForm->baseWidget(),
                    XmNwidth, 234, 
                    XmNheight, 298, 
                    (XtPointer) NULL );

    // Add the component as the main view

    addView (_mainForm);
    _mainForm->setParent(this);

    // Create the panes of this window's menubar. The menubar itself
    //  is created automatically by ViewKit


    // The filePane menu pane

    _filePane =  addMenuPane("filePane");

    _exitButton =  _filePane->addAction ( "exitButton", 
                                        &MainWindowMainWindow::quitCallback, 
                                        (XtPointer) this );

    // The editPane menu pane

    _editPane =  addMenuPane("editPane");

    _optionsButton =  _editPane->addAction ( "optionsButton", 
                                        &MainWindowMainWindow::optionsCallback, 
                                        (XtPointer) this );

    // The viewPane menu pane

    _viewPane =  addMenuPane("viewPane");

    _audioPanelButton =  _viewPane->addAction ( "audioPanelButton", 
                                        &MainWindowMainWindow::audioPanelCallback, 
                                        (XtPointer) this );

    _statButton =  _viewPane->addAction ( "statButton", 
                                        &MainWindowMainWindow::statisticsCallback, 
                                        (XtPointer) this );


     VkConfigureWM ( this,
                    TRUE,  //quit
                    FALSE,  // close
                    TRUE,   // border
                    TRUE,   // title
                    FALSE,  // allow resize
                    TRUE,   // allow iconify
                    TRUE);  // Show window manager menu

    //---- Start editable code block: MainWindowMainWindow constructor

    session_control_ = BV_Session_Control::instance();

    /*
    reactor_ = new Reactor();
    acceptor_ = new Acceptor <BV_Svc_Handler, SOCK_Acceptor>();

    struct sockaddr_in addr;
    bzero(&addr, sizeof(addr));
    addr.sin_family = AF_INET;
    addr.sin_addr.s_addr = htonl(INADDR_ANY);
    addr.sin_port = htons(BV_PORT);
    
    session_control_->set_reactor(reactor_);
    
    if(acceptor_->open(&addr, reactor_) < 0)
      {
	printf("BuenaVista already started, exitting...\n");
	exit(-1);
      }
    */    

    session_control_->setGUI(this, _mainForm);

    audio_app_ = new BV_AudioApp();

    video_app_ = new BV_VideoApp();

    tango_ctrl_ = new BV_Tango_Control((int)argCount, (char **)args);
    audio_app_->init();
    video_app_->init();
    //---- End editable code block: MainWindowMainWindow constructor


} // End Constructor


MainWindowMainWindow::~MainWindowMainWindow()
{
  //delete _mainForm;
    //---- Start editable code block: MainWindowMainWindow destructor
    
    audio_app_->close();
    delete audio_app_;
    video_app_->close();
    //sleep(1);
     delete video_app_;
     XtRemoveWorkProc(ctrlWorkProcHandle);
    //reactor_->close();
    //delete reactor_;
     delete _mainForm;
     delete tango_ctrl_;
    session_control_->destroy();

    //---- End editable code block: MainWindowMainWindow destructor
} // End destructor


const char *MainWindowMainWindow::className()
{
    return ("MainWindowMainWindow");
} // End className()


Boolean MainWindowMainWindow::okToQuit()
{    //---- Start editable code block: MainWindowMainWindow okToQuit



    // This member function is called when the user quits by calling
    // theApplication->terminate() or uses the window manager close protocol
    // This function can abort the operation by returning FALSE, or do some.
    // cleanup before returning TRUE. The actual decision is normally passed on
    // to the view object


    // Query the view object, and give it a chance to cleanup

  audio_app_->close();
  delete audio_app_;
  video_app_->close();
  delete video_app_;
  XtRemoveWorkProc(ctrlWorkProcHandle);
  //reactor_->reset();
  //reactor_->close();
  //delete reactor_;
  delete _mainForm;
  session_control_->destroy();
  exit(0);
  return ( _mainForm->okToQuit() );

    //---- End editable code block: MainWindowMainWindow okToQuit
} // End okToQuit()



/////////////////////////////////////////////////////////////// 
// The following functions are static member functions used to 
// interface with Motif.
/////////////////////////////////// 

void MainWindowMainWindow::audioPanelCallback ( Widget    w,
                                                XtPointer clientData,
                                                XtPointer callData ) 
{ 
    MainWindowMainWindow* obj = ( MainWindowMainWindow * ) clientData;
    obj->audioPanel ( w, callData );
}

void MainWindowMainWindow::chatCallback ( Widget    w,
                                          XtPointer clientData,
                                          XtPointer callData ) 
{ 
    MainWindowMainWindow* obj = ( MainWindowMainWindow * ) clientData;
    obj->chat ( w, callData );
}

void MainWindowMainWindow::confCallback ( Widget    w,
                                          XtPointer clientData,
                                          XtPointer callData ) 
{ 
    MainWindowMainWindow* obj = ( MainWindowMainWindow * ) clientData;
    obj->conf ( w, callData );
}

void MainWindowMainWindow::optionsCallback ( Widget    w,
                                             XtPointer clientData,
                                             XtPointer callData ) 
{ 
    MainWindowMainWindow* obj = ( MainWindowMainWindow * ) clientData;
    obj->options ( w, callData );
}

void MainWindowMainWindow::overviewCallback ( Widget    w,
                                              XtPointer clientData,
                                              XtPointer callData ) 
{ 
    MainWindowMainWindow* obj = ( MainWindowMainWindow * ) clientData;
    obj->overview ( w, callData );
}

void MainWindowMainWindow::quitCallback ( Widget    w,
                                          XtPointer clientData,
                                          XtPointer callData ) 
{ 
    MainWindowMainWindow* obj = ( MainWindowMainWindow * ) clientData;
    obj->quit ( w, callData );
}

void MainWindowMainWindow::resetCallback ( Widget    w,
                                           XtPointer clientData,
                                           XtPointer callData ) 
{ 
    MainWindowMainWindow* obj = ( MainWindowMainWindow * ) clientData;
    obj->reset ( w, callData );
}

void MainWindowMainWindow::statisticsCallback ( Widget    w,
                                                XtPointer clientData,
                                                XtPointer callData ) 
{ 
    MainWindowMainWindow* obj = ( MainWindowMainWindow * ) clientData;
    obj->statistics ( w, callData );
}

void MainWindowMainWindow::usersCallback ( Widget    w,
                                           XtPointer clientData,
                                           XtPointer callData ) 
{ 
    MainWindowMainWindow* obj = ( MainWindowMainWindow * ) clientData;
    obj->users ( w, callData );
}





/////////////////////////////////////////////////////////////// 
// The following functions are called from the menu items 
// in this window.
/////////////////////////////////// 

void MainWindowMainWindow::audioPanel ( Widget w, XtPointer callData ) 
{
    //---- Start editable code block: MainWindowMainWindow audioPanel


    // This virtual function is called from audioPanelCallback.
    // By default this member function passes control
    // to the component contained by this window

    _mainForm->audioPanel( w, callData);


    //---- End editable code block: MainWindowMainWindow audioPanel

} // End MainWindowMainWindow::audioPanel()


void MainWindowMainWindow::chat ( Widget w, XtPointer callData ) 
{
    //---- Start editable code block: MainWindowMainWindow chat


    // This virtual function is called from chatCallback.
    // By default this member function passes control
    // to the component contained by this window

    _mainForm->chat( w, callData);


    //---- End editable code block: MainWindowMainWindow chat

} // End MainWindowMainWindow::chat()


void MainWindowMainWindow::conf ( Widget w, XtPointer callData ) 
{
    //---- Start editable code block: MainWindowMainWindow conf


    // This virtual function is called from confCallback.
    // By default this member function passes control
    // to the component contained by this window

    _mainForm->conf( w, callData);


    //---- End editable code block: MainWindowMainWindow conf

} // End MainWindowMainWindow::conf()


void MainWindowMainWindow::options ( Widget w, XtPointer callData ) 
{
    //---- Start editable code block: MainWindowMainWindow options


    // This virtual function is called from optionsCallback.
    // By default this member function passes control
    // to the component contained by this window

    _mainForm->options( w, callData);


    //---- End editable code block: MainWindowMainWindow options

} // End MainWindowMainWindow::options()


void MainWindowMainWindow::overview ( Widget w, XtPointer callData ) 
{
    //---- Start editable code block: MainWindowMainWindow overview


    // This virtual function is called from overviewCallback.
    // By default this member function passes control
    // to the component contained by this window

    _mainForm->overview( w, callData);


    //---- End editable code block: MainWindowMainWindow overview

} // End MainWindowMainWindow::overview()


void MainWindowMainWindow::quit ( Widget, XtPointer ) 
{
    // Exit via quitYourself() to allow the application
    // to go through its normal shutdown routines, checking with
    // all windows, views, and so on
  
  audio_app_->close();
  delete audio_app_;
  video_app_->close();
  delete video_app_;
  XtRemoveWorkProc(ctrlWorkProcHandle);
  //reactor_->reset();
  //reactor_->close();
  //delete reactor_;
  delete _mainForm;
  delete tango_ctrl_;
  session_control_->destroy();
  exit(0);
  //  theApplication->quitYourself();

}  // End MainWindowMainWindow::quit()

void MainWindowMainWindow::reset ( Widget w, XtPointer callData ) 
{
  //---- Start editable code block: MainWindowMainWindow reset
  
  
  // This virtual function is called from resetCallback.
  // By default this member function passes control
  // to the component contained by this window
  
  //reactor_->reset();
  session_control_->reset();
  
  _mainForm->reset( w, callData);


  //---- End editable code block: MainWindowMainWindow reset

} // End MainWindowMainWindow::reset()


void MainWindowMainWindow::statistics ( Widget w, XtPointer callData ) 
{
    //---- Start editable code block: MainWindowMainWindow statistics


    // This virtual function is called from statisticsCallback.
    // By default this member function passes control
    // to the component contained by this window

    _mainForm->statistics( w, callData);


    //---- End editable code block: MainWindowMainWindow statistics

} // End MainWindowMainWindow::statistics()


void MainWindowMainWindow::users ( Widget w, XtPointer callData ) 
{
    //---- Start editable code block: MainWindowMainWindow users


    // This virtual function is called from usersCallback.
    // By default this member function passes control
    // to the component contained by this window

    _mainForm->users( w, callData);


    //---- End editable code block: MainWindowMainWindow users

} // End MainWindowMainWindow::users()





//---- Start editable code block: End of generated code

Reactor *MainWindowMainWindow::getReactor()
{
  return reactor_;
}

void MainWindowMainWindow::playVideo()
{
  video_app_->play();
}

void MainWindowMainWindow::stopVideo()
{
  video_app_->stop();
}

void MainWindowMainWindow::playAudio()
{
  audio_app_->play();
}

void MainWindowMainWindow::stopAudio()
{
  audio_app_->stop();
}

void MainWindowMainWindow::updateOptions(Options *options)
{
  audio_app_->update_options(options);
  video_app_->update_options(options);
}

void MainWindowMainWindow::setMicCutoffLevel(int level)
{
  Options *opt = session_control_->get_options();
  opt->mic_cutoff = level;
  audio_app_->update_options(opt);  
}

//---- End editable code block: End of generated code


